@@ -59,7 +59,9 @@
 
 #include <X11/Xos.h>                /* For gettimeofday */
 #include "dixstruct.h"
+#ifdef PANORAMIX
 #include "panoramiXsrv.h"
+#endif
 
 #include <signal.h>             /* For SIGQUIT */
 
@@ -508,7 +510,7 @@ static void dmxDisplayInit(DMXScreenInfo *dmxScreen)
 /* If this doesn't compile, just add || defined(yoursystem) to the line
  * below.  This information is to help with bug reports and is not
  * critical. */
-#if !defined(_POSIX_SOURCE) && !defined(__sgi)
+#if !defined(_POSIX_SOURCE) 
 static const char *dmxExecOS(void) { return ""; }
 #else
 #include <sys/utsname.h>
@@ -538,14 +540,6 @@ static const char *dmxBuildCompiler(void)
 #if defined(__GNUC__) && defined(__GNUC_MINOR__) &&defined(__GNUC_PATCHLEVEL__)
         XmuSnprintf(buffer, sizeof(buffer)-1, "gcc %d.%d.%d",
                     __GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__);
-#elif defined(__sgi) && defined(_COMPILER_VERSION) && !defined(__GNUC__)
-        {
-            int a = _COMPILER_VERSION / 100;
-            int b = (_COMPILER_VERSION - a * 100) / 10;
-            int c = _COMPILER_VERSION - a * 100 - b * 10;
-            XmuSnprintf(buffer, sizeof(buffer)-1, "SGI MIPSpro %d.%d.%d",
-                        a, b, c);
-        }
 #endif
     }
     return buffer;
@@ -746,8 +740,7 @@ void InitOutput(ScreenInfo *pScreenInfo, int argc, char *argv[])
 		nconfigs = dmxScreen->numGlxVisuals;
 	    }
 
-	    configprivs = xalloc(dmxScreen->beNumVisuals *
-				 sizeof(dmxGlxVisualPrivate*));
+	    configprivs = xalloc(nconfigs * sizeof(dmxGlxVisualPrivate*));
 
 	    if (configs != NULL && configprivs != NULL) {
 
@@ -779,6 +772,8 @@ void InitOutput(ScreenInfo *pScreenInfo, int argc, char *argv[])
 
 		/* Hand out the glx configs to glx extension */
 		GlxSetVisualConfigs(nconfigs, configs, (void**)configprivs);
+
+                XFlush(dmxScreen->beDisplay);
 	    }
 	}
 #endif  /* GLXEXT */
@@ -873,13 +868,6 @@ void OsVendorFatalError(void)
 {
 }
 
-/** This funciton is called by InitGlobals from Xserver/os/utils.c to
- * initialize any ddx specific globals at a very early point in the
- * server startup. */
-void ddxInitGlobals(void)
-{
-}
-
 /** Process our command line arguments. */
 int ddxProcessArgument(int argc, char *argv[], int i)
 {
